# -*- coding: utf-8 -*-

import unittest
from hotxlfp import Parser


class TestStatistical(unittest.TestCase):

    def test_error_type(self):
        p = Parser(debug=True)
        ret = p.parse('ERROR.TYPE(1/0)')
        self.assertEqual(ret['result'], 2)
        self.assertEqual(ret['error'], None)

    def test_iserr(self):
        p = Parser(debug=True)
        ret = p.parse('ISERR(1/0)')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['error'], None)

    def test_iserror(self):
        p = Parser(debug=True)
        ret = p.parse('ISERROR(1/0)')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['error'], None)
        p = Parser(debug=True)
        ret = p.parse('ISERROR(1/2)')
        self.assertEqual(ret['result'], False)
        self.assertEqual(ret['error'], None)

    def test_iseven(self):
        p = Parser(debug=True)
        ret = p.parse('ISEVEN(1)')
        self.assertEqual(ret['result'], False)
        self.assertEqual(ret['error'], None)
        ret = p.parse('ISEVEN(-1)')
        self.assertEqual(ret['result'], False)
        self.assertEqual(ret['error'], None)
        ret = p.parse('ISEVEN(2)')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['error'], None)
        ret = p.parse('ISEVEN(-2)')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['error'], None)
        ret = p.parse('ISEVEN(1000)')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['error'], None)
        ret = p.parse('ISEVEN("A")')
        self.assertEqual(ret['result'], None)
        self.assertEqual(ret['error'], '#VALUE!')
        ret = p.parse('ISEVEN(2.21)')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['error'], None)

    def test_isodd(self):
        p = Parser(debug=True)
        ret = p.parse('ISODD(1)')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['error'], None)
        ret = p.parse('ISODD(-1)')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['error'], None)
        ret = p.parse('ISODD(2)')
        self.assertEqual(ret['result'], False)
        self.assertEqual(ret['error'], None)
        ret = p.parse('ISODD(-2)')
        self.assertEqual(ret['result'], False)
        self.assertEqual(ret['error'], None)
        ret = p.parse('ISODD(1000)')
        self.assertEqual(ret['result'], False)
        self.assertEqual(ret['error'], None)
        ret = p.parse('ISODD("A")')
        self.assertEqual(ret['result'], None)
        self.assertEqual(ret['error'], '#VALUE!')
        ret = p.parse('ISODD(2.21)')
        self.assertEqual(ret['result'], False)
        self.assertEqual(ret['error'], None)

    def test_istext(self):
        p = Parser(debug=True)
        ret = p.parse('ISTEXT("foo")')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['error'], None)
        ret = p.parse('ISTEXT(NA())')
        self.assertEqual(ret['result'], False)
        self.assertEqual(ret['error'], None)
        ret = p.parse('ISTEXT(1)')
        self.assertEqual(ret['result'], False)
        self.assertEqual(ret['error'], None)

    def test_isnumber(self):
        p = Parser(debug=True)
        ret = p.parse('ISNUMBER("foo")')
        self.assertEqual(ret['result'], False)
        self.assertEqual(ret['error'], None)
        ret = p.parse('ISNUMBER(TRUE)')
        self.assertEqual(ret['result'], False)
        self.assertEqual(ret['error'], None)
        ret = p.parse('ISNUMBER(NA())')
        self.assertEqual(ret['result'], False)
        self.assertEqual(ret['error'], None)
        ret = p.parse('ISNUMBER(1)')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['error'], None)

    def test_islogical(self):
        p = Parser(debug=True)
        ret = p.parse('ISLOGICAL("foo")')
        self.assertEqual(ret['result'], False)
        self.assertEqual(ret['error'], None)
        ret = p.parse('ISLOGICAL(TRUE)')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['error'], None)
        ret = p.parse('ISLOGICAL(NA())')
        self.assertEqual(ret['result'], False)
        self.assertEqual(ret['error'], None)
        ret = p.parse('ISLOGICAL(1)')
        self.assertEqual(ret['result'], False)
        self.assertEqual(ret['error'], None)

    def test_isna(self):
        p = Parser(debug=True)
        ret = p.parse('ISNA(NA())')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['error'], None)
        ret = p.parse('ISNA(1/0)')
        self.assertEqual(ret['result'], False)
        self.assertEqual(ret['error'], None)

    def test_isnontext(self):
        p = Parser(debug=True)
        ret = p.parse('ISNONTEXT("foo")')
        self.assertEqual(ret['result'], False)
        self.assertEqual(ret['error'], None)
        ret = p.parse('ISNONTEXT(NA())')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['error'], None)
        ret = p.parse('ISNONTEXT(1)')
        self.assertEqual(ret['result'], True)
        self.assertEqual(ret['error'], None)

    def test_n(self):
        p = Parser(debug=True)
        ret = p.parse('N(1)')
        self.assertEqual(ret['result'], 1)
        self.assertEqual(ret['error'], None)
        ret = p.parse('N(TODAY())')
        ret2 = p.parse('DATEVALUE(TODAY())')
        self.assertEqual(ret['result'], ret2['result'])
        self.assertEqual(ret['error'], None)
        ret = p.parse('N("1024")')
        self.assertEqual(ret['result'], 0)
        self.assertEqual(ret['error'], None)
        ret = p.parse('N("foo")')
        self.assertEqual(ret['result'], 0)
        self.assertEqual(ret['error'], None)
        ret = p.parse('N(TRUE())')
        self.assertEqual(ret['result'], 1)
        self.assertEqual(ret['error'], None)
        ret = p.parse('N(FALSE())')
        self.assertEqual(ret['result'], 0)
        self.assertEqual(ret['error'], None)

    def test_t(self):
        p = Parser(debug=True)
        ret = p.parse('T(100)')
        self.assertEqual(ret['result'], '')
        self.assertEqual(ret['error'], None)
        ret = p.parse('T(TODAY())')
        self.assertEqual(ret['result'], '')
        self.assertEqual(ret['error'], None)
        ret = p.parse('T("1024")')
        self.assertEqual(ret['result'], "1024")
        self.assertEqual(ret['error'], None)
        ret = p.parse('T("foo")')
        self.assertEqual(ret['result'], "foo")
        self.assertEqual(ret['error'], None)
        ret = p.parse('T(TRUE())')
        self.assertEqual(ret['result'], '')
        self.assertEqual(ret['error'], None)
        ret = p.parse('T(FALSE())')
        self.assertEqual(ret['result'], '')
        self.assertEqual(ret['error'], None)
        ret = p.parse('T(10%)')
        self.assertEqual(ret['result'], '')
        self.assertEqual(ret['error'], None)
        ret = p.parse('T("FOO")')
        self.assertEqual(ret['result'], "FOO")
        self.assertEqual(ret['error'], None)